/* SPDX-License-Identifier: GPL-2.0 or MIT */
#ifndef _ASM_X86_VMWARE_H
#define _ASM_X86_VMWARE_H

#include <linux/bits.h>
#include <asm/cpufeatures.h>
#include <asm/alternative.h>
#include <linux/stringify.h>

#define CPUID_VMWARE_INFO_LEAF               0x40000000
#define CPUID_VMWARE_FEATURES_LEAF           0x40000010
#define CPUID_VMWARE_FEATURES_ECX_VMMCALL    BIT(0)
#define CPUID_VMWARE_FEATURES_ECX_VMCALL     BIT(1)

/*
 * The hypercall definitions differ in the low word of the %edx argument
 * in the following way: the old port base interface uses the port
 * number to distinguish between high- and low bandwidth versions.
 *
 * The new vmcall interface instead uses a set of flags to select
 * bandwidth mode and transfer direction. The flags should be loaded
 * into %dx by any user and are automatically replaced by the port
 * number if the VMWARE_HYPERVISOR_PORT method is used.
 *
 * In short, new driver code should strictly use the new definition of
 * %dx content.
 */

/* Old port-based version */
#define VMWARE_HYPERVISOR_PORT    0x5658
#define VMWARE_HYPERVISOR_PORT_HB 0x5659

/* Current vmcall / vmmcall version */
#define VMWARE_HYPERVISOR_HB   BIT(0)
#define VMWARE_HYPERVISOR_OUT  BIT(1)

/* The low bandwidth call. The low word of edx is presumed clear. */
#define VMWARE_HYPERCALL						\
	ALTERNATIVE_3("callq vmw_hypercall_slowpath",			\
		      "movw $" __stringify(VMWARE_HYPERVISOR_PORT)	\
							", %%dx; "	\
		      "inl (%%dx), %%eax", X86_FEATURE_HYPERVISOR,	\
		      "vmcall", X86_FEATURE_VMCALL,			\
		      "vmmcall", X86_FEATURE_VMW_VMMCALL)

/*
 * The high bandwidth out call.
 */
#define VMWARE_HYPERCALL_HB_OUT						\
	ALTERNATIVE_3("callq vmw_hypercall_hb_out_slowpath",		\
		      "movw $" __stringify(VMWARE_HYPERVISOR_PORT_HB)	\
							", %%dx; "	\
		      "cld; rep outsb", X86_FEATURE_HYPERVISOR,		\
		      "movw $" __stringify(VMWARE_HYPERVISOR_HB |	\
					   VMWARE_HYPERVISOR_OUT)	\
							", %%dx; "	\
		      "vmcall", X86_FEATURE_VMCALL,			\
		      "movw $" __stringify(VMWARE_HYPERVISOR_HB |	\
					   VMWARE_HYPERVISOR_OUT)	\
							", %%dx; "	\
		      "vmmcall", X86_FEATURE_VMW_VMMCALL)

/*
 * The high bandwidth in call.
 */
#define VMWARE_HYPERCALL_HB_IN						\
	ALTERNATIVE_3("callq vmw_hypercall_hb_in_slowpath",		\
		      "movw $" __stringify(VMWARE_HYPERVISOR_PORT_HB)	\
							", %%dx; "	\
		      "cld; rep insb", X86_FEATURE_HYPERVISOR,		\
		      "movw $" __stringify(VMWARE_HYPERVISOR_HB)	\
							", %%dx; "	\
		      "vmcall", X86_FEATURE_VMCALL,			\
		      "movw $" __stringify(VMWARE_HYPERVISOR_HB)	\
							", %%dx; "	\
		      "vmmcall", X86_FEATURE_VMW_VMMCALL)
#endif
